@@ -2,7 +2,7 @@
 # encoding: utf-8
 
 """
-An RFC 2821 smtp proxy Port from Python Standrad Library for gevent usage
+An RFC 2821 smtp proxy server port from Python Standrad Library for Gevent usage
 """
 
 # Overview:
@@ -21,7 +21,7 @@ An RFC 2821 smtp proxy Port from Python Standrad Library for gevent usage
 #   SMTP errors from the backend server at all.  This should be fixed
 #   (contributions are welcome!).
 #
-# Please note that this script requires Python 2.0
+# Please note that this script requires Python 2.7
 #
 # Author: Barry Warsaw <barry@python.org>
 #         Meng Zhuo <mengzhuo1203@gmail.com>
@@ -41,12 +41,13 @@ from gevent import socket, monkey, sleep, Timeout
 monkey.patch_all()
 from gevent.server import StreamServer
 
+
 import ssl
 from ssl import CERT_NONE
 
-from channel import SMTPChannel
+from .channel import SMTPChannel
 
-__all__ = ["SMTPServer", "DebuggingServer", "PureProxy"]
+__all__ = ["SMTPServer", "DebuggingServer", "PureProxy", 'SSLSettings']
 
 class ConnectionTimeout(Exception):
     pass
@@ -56,12 +57,16 @@ EMPTYSTRING = ''
 COMMASPACE = ', '
 
 class SSLSettings(UserDict):
-
+    """SSL settings object"""
     def __init__(self, keyfile=None, certfile=None,
                  ssl_version='PROTOCOL_SSLv23', ca_certs=None,
                  do_handshake_on_connect=True, cert_reqs=CERT_NONE,
                  suppress_ragged_eofs=True, ciphers=None, **kwargs):
+        """settings of SSL
 
+        :param keyfile: SSL key file path usally end with ".key"
+        :param certfile: SSL cert file path usally end with ".crt"
+        """
         UserDict.__init__(self) 
         self.data.update( dict(keyfile = keyfile,
                                 certfile = certfile,
@@ -75,10 +80,19 @@ class SSLSettings(UserDict):
 
 
 class SMTPServer(StreamServer):
-    
-    
-    def __init__(self, localaddr=None, remoteaddr=None, timeout=60,
-                       ssl=False, **kwargs):
+    """Abstrcted SMTP server
+    """
+
+    def __init__(self, localaddr=None, remoteaddr=None, 
+                 timeout=60, data_size_limit=10240000, **kwargs):
+        """Initialize SMTP Server
+
+        :param localaddr: tuple pair that start server, like `('127.0.0.1', 25)`
+        :param remoteaddr: ip address (string or list) that can relay on this server
+        :param timeout: int that connection Timeout
+        :param data_size_limit: max byte per mail data
+        :param kwargs: other key-arguments will pass to :class:`.SSLSettings`
+        """
 
         self.relay = bool(remoteaddr)
         self.remoteaddr = remoteaddr
@@ -88,16 +102,14 @@ class SMTPServer(StreamServer):
         if not self.localaddr:
             self.localaddr = ('127.0.0.1', 25)
         
-        self.ssl = ssl
-
-        if self.ssl:
-            self.ssl = SSLSettings(**kwargs)
+        self.ssl = None
         
         self.timeout = int(timeout)
 
-        self.data_size_limit = 1024000
-        if 'data_size_limit' in kwargs:
-            self.data_size_limit = int(kwargs.pop('data_size_limit'))
+        self.data_size_limit = int(data_size_limit)
+
+        if 'keyfile' in kwargs:
+            self.ssl = SSLSettings(**kwargs)
 
         super(SMTPServer, self).__init__(self.localaddr, self.handle)
 
@@ -113,7 +125,7 @@ class SMTPServer(StreamServer):
                     sc = SMTPChannel(self, sock, addr, self.data_size_limit)
                     while not sc.closed:
                         sc.handle_read()
-                        sleep(0.1) # relieve CPU
+                        sleep(0.0001) # relieve CPU
                 finally:
                     sc.close_when_done()
 
@@ -128,19 +140,14 @@ class SMTPServer(StreamServer):
     def process_message(self, peer, mailfrom, rcpttos, data):
         """Override this abstract method to handle messages from the client.
 
-        peer is a tuple containing (ipaddr, port) of the client that made the
-        socket connection to our smtp port.
-
-        mailfrom is the raw address the client claims the message is coming
-        from.
-
-        rcpttos is a list of raw addresses the client wishes to deliver the
-        message to.
-
-        data is a string containing the entire full text of the message,
-        headers (if supplied) and all.  It has been `de-transparencied'
-        according to RFC 821, Section 4.5.2.  In other words, a line
-        containing a `.' followed by other text has had the leading dot
+        :param peer: is a tuple containing (ipaddr, port) of the client that made\n
+                     the socket connection to our smtp port.
+        :param mailfrom: is the raw address the client claims the message is coming from.
+        :param rcpttos: is a list of raw addresses the client wishes to deliver the message to.
+        :param data: is a string containing the entire full text of the message,\n
+                     headers (if supplied) and all.  It has been `de-transparencied'\n
+                     according to RFC 821, Section 4.5.2.\n
+                     In other words, a line containing a `.' followed by other text has had the leading dot
         removed.
 
         This function should return None, for a normal `250 Ok' response;
@@ -148,7 +155,17 @@ class SMTPServer(StreamServer):
 
         """
         raise NotImplementedError
+    
+    # API that handle rcpt
+    def process_rcpt(self, address):
+        """Override this abstract method to handle rcpt from the client
 
+        :param address: is raw address the client wishes to deliver the message to
+
+        This function should return None, for a normal `250 Ok' response;
+        otherwise it returns the desired response string in RFC 821 format.
+        """
+        pass
 
 class DebuggingServer(SMTPServer):
     # Do something with the gathered message
